<?php

class ApiClient
{
    protected $results = array();
    protected $config;

    public function __construct($config)
    {
        $this->config = $config;
    }


    /**
     * Make external API call to registrar API.
     *
     * @param string $action
     * @param array $postfields
     *
     * @return array
     * @throws \Exception Bad API response
     *
     * @throws \Exception Connection error
     */
    public function request($action, $postfields)
    {
        $timestamp = time();
        $params = [
            'user' => $this->config['user'],
            'timestamp' => $timestamp,
            'token' => md5($this->config['user'] . md5($this->config['pass']) . $timestamp . $action),
            'apikey' => $this->config['apikey'],
            'from' => 'WHMCS'
        ];
        $params = array_merge($params, $postfields);

        $ch = curl_init();
        $url = 'https://' . $this->config['server'] . '/api4/domain/index.php?command=' . $action;

        curl_setopt($ch, CURLOPT_URL, $url);
        $this->writeLog('REQUEST:' . $url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($params));
        $this->writeLog('REQUEST DATA:' . http_build_query($params));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
        curl_setopt($ch, CURLOPT_TIMEOUT, $this->config['timeout']);
        $response = curl_exec($ch);
        if (curl_errno($ch)) {
            $msg = 'Connection Error: ' . curl_errno($ch) . ' - ' . curl_error($ch);
            $this->writeLog('RESPONSE:' . $msg);
            throw new \Exception($msg);
        }
        curl_close($ch);
        $this->writeLog('RESPONSE:' . $response);

        $this->results = $this->processResponse($response);

        logModuleCall(
            'OnlineNICAPI4',
            $action,
            $postfields,
            $response,
            $this->results,
            array(
                $this->config['user'],
                $this->config['pass'],
            )
        );

        if ($this->results === null && json_last_error() !== JSON_ERROR_NONE) {
            throw new \Exception('Bad response received from API');
        }
        if ($this->results['code'] > 1000 && $this->results['code'] < 2000) {
            throw new \Exception($this->results['msg']);
        }

        return $this->results;
    }

    /**
     * Process API response.
     *
     * @param string $response
     *
     * @return array
     */
    public function processResponse($response)
    {
        return json_decode($response, true);
    }

    /**
     * Get from response results.
     *
     * @param string $key
     *
     * @return string
     */
    public function getFromResponse($key)
    {
        return isset($this->results[$key]) ? $this->results[$key] : '';
    }

    /**
     * Write Log
     *
     * @param $str
     *
     * @return null
     */
    private function writeLog($str)
    {
        if (!$this->config['log_record']) {
            return;
        }
        $filename = $this->config['log_path'] . 'api_client-' . date('Ymd') . '.log';
        $fp = fopen($filename, 'a+');
        if ($fp) {
            fwrite($fp, date('Y-m-d H:i:s') . ' ' . $str . "\r\n");
            fclose($fp);
        }
        chmod($filename, 0666);
        return;
    }
}

